@@ -6,7 +6,6 @@ package benchmark
 
 import (
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
 	"os/exec"
@@ -22,42 +21,21 @@ import (
 	"github.com/hanwen/go-fuse/v2/internal/testutil"
 )
 
-func setupFs(node fs.InodeEmbedder, N int) (string, func()) {
+func setupFS(node fs.InodeEmbedder, N int, tb testing.TB) string {
 	opts := &fs.Options{}
 	opts.Debug = testutil.VerboseTest()
-	mountPoint, err := os.MkdirTemp("", "")
-	if err != nil {
-		log.Panicf("TempDir: %v", err)
-	}
+	mountPoint := tb.TempDir()
 	server, err := fs.Mount(mountPoint, node, opts)
 	if err != nil {
-		log.Panicf("cannot mount %v", err)
+		tb.Fatalf("cannot mount %v", err)
 	}
-	lmap := NewLatencyMap()
-	if testutil.VerboseTest() {
-		server.RecordLatencies(lmap)
-	}
-	return mountPoint, func() {
-		if testutil.VerboseTest() {
-			var total time.Duration
-			for _, n := range []string{"LOOKUP", "GETATTR", "OPENDIR", "READDIR",
-				"READDIRPLUS", "RELEASEDIR", "FLUSH",
-			} {
-				if count, dt := lmap.Get(n); count > 0 {
-					total += dt
-					log.Printf("%s %v/call n=%d", n,
-						dt/time.Duration(count), count)
-				}
-			}
-
-			log.Printf("total %v, %v/bench op", total, total/time.Duration(N))
-		}
-
+	tb.Cleanup(func() {
 		err := server.Unmount()
 		if err != nil {
 			log.Println("error during unmount", err)
 		}
-	}
+	})
+	return mountPoint
 }
 
 func TestNewStatFs(t *testing.T) {
@@ -68,10 +46,9 @@ func TestNewStatFs(t *testing.T) {
 		fs.AddFile(n, fuse.Attr{Mode: syscall.S_IFREG})
 	}
 
-	wd, clean := setupFs(fs, 1)
-	defer clean()
+	wd := setupFS(fs, 1, t)
 
-	names, err := ioutil.ReadDir(wd)
+	names, err := os.ReadDir(wd)
 	if err != nil {
 		t.Fatalf("failed: %v", err)
 	}
@@ -86,14 +63,14 @@ func TestNewStatFs(t *testing.T) {
 	if !fi.IsDir() {
 		t.Error("mode", fi)
 	}
-	names, err = ioutil.ReadDir(wd + "/sub")
+	names, err = os.ReadDir(wd + "/sub")
 	if err != nil {
 		t.Fatalf("failed: %v", err)
 	}
 	if len(names) != 2 {
 		t.Error("readdir /sub", names)
 	}
-	names, err = ioutil.ReadDir(wd + "/sub/dir")
+	names, err = os.ReadDir(wd + "/sub/dir")
 	if err != nil {
 		t.Fatalf("failed: %v", err)
 	}
@@ -110,7 +87,7 @@ func TestNewStatFs(t *testing.T) {
 	}
 }
 
-func BenchmarkGoFuseStat(b *testing.B) {
+func BenchmarkGoFSStat(b *testing.B) {
 	b.StopTimer()
 	fs := &StatFS{}
 
@@ -121,15 +98,14 @@ func BenchmarkGoFuseStat(b *testing.B) {
 		fs.AddFile(fn, fuse.Attr{Mode: syscall.S_IFREG})
 	}
 
-	wd, clean := setupFs(fs, b.N)
-	defer clean()
+	mnt := setupFS(fs, b.N, b)
 
 	for i, l := range files {
-		files[i] = filepath.Join(wd, l)
+		files[i] = filepath.Join(mnt, l)
 	}
 
 	threads := runtime.GOMAXPROCS(0)
-	if err := TestingBOnePass(b, threads, fileList, wd); err != nil {
+	if err := TestingBOnePass(b, threads, fileList, mnt); err != nil {
 		b.Fatalf("TestingBOnePass %v8", err)
 	}
 }
@@ -145,7 +121,7 @@ func readdir(d string) error {
 	return f.Close()
 }
 
-func BenchmarkGoFuseReaddir(b *testing.B) {
+func BenchmarkGoFSReaddir(b *testing.B) {
 	b.StopTimer()
 	fs := &StatFS{}
 
@@ -157,12 +133,11 @@ func BenchmarkGoFuseReaddir(b *testing.B) {
 		dirSet[filepath.Dir(fn)] = struct{}{}
 	}
 
-	wd, clean := setupFs(fs, b.N)
-	defer clean()
+	mnt := setupFS(fs, b.N, b)
 
 	var dirs []string
 	for dir := range dirSet {
-		dirs = append(dirs, filepath.Join(wd, dir))
+		dirs = append(dirs, filepath.Join(mnt, dir))
 	}
 	b.StartTimer()
 	todo := b.N
@@ -213,14 +188,7 @@ func TestingBOnePass(b *testing.B, threads int, filelist, mountPoint string) err
 	return nil
 }
 
-// Add this so we can estimate impact on latency numbers.
-func BenchmarkTimeNow(b *testing.B) {
-	for i := 0; i < b.N; i++ {
-		time.Now()
-	}
-}
-
-func BenchmarkCFuseThreadedStat(b *testing.B) {
+func BenchmarkLibfuseHighlevelThreadedStat(b *testing.B) {
 	b.StopTimer()
 
 	wd, _ := os.Getwd()
@@ -242,7 +210,7 @@ func BenchmarkCFuseThreadedStat(b *testing.B) {
 		out = append(out, k)
 	}
 
-	f, err := ioutil.TempFile("", "")
+	f, err := os.CreateTemp("", "")
 	if err != nil {
 		b.Fatalf("failed: %v", err)
 	}
