@@ -6,92 +6,18 @@ package fs
 
 import (
 	"bytes"
-	"io/ioutil"
 	"os"
-	"reflect"
 	"sync"
 	"syscall"
 	"testing"
 	"time"
+	"unsafe"
 
 	"github.com/hanwen/go-fuse/v2/fuse"
 	"github.com/hanwen/go-fuse/v2/internal/testutil"
-	"github.com/kylelemons/godebug/pretty"
 	"golang.org/x/sys/unix"
 )
 
-func TestRenameExchange(t *testing.T) {
-	tc := newTestCase(t, &testOptions{attrCache: true, entryCache: true})
-
-	if err := os.Mkdir(tc.origDir+"/dir", 0755); err != nil {
-		t.Fatalf("Mkdir: %v", err)
-	}
-	tc.writeOrig("file", "hello", 0644)
-	tc.writeOrig("dir/file", "x", 0644)
-
-	f1, err := syscall.Open(tc.mntDir+"/", syscall.O_DIRECTORY, 0)
-	if err != nil {
-		t.Fatalf("open 1: %v", err)
-	}
-	defer syscall.Close(f1)
-	f2, err := syscall.Open(tc.mntDir+"/dir", syscall.O_DIRECTORY, 0)
-	if err != nil {
-		t.Fatalf("open 2: %v", err)
-	}
-	defer syscall.Close(f2)
-
-	var before1, before2 unix.Stat_t
-	if err := unix.Fstatat(f1, "file", &before1, 0); err != nil {
-		t.Fatalf("Fstatat: %v", err)
-	}
-	if err := unix.Fstatat(f2, "file", &before2, 0); err != nil {
-		t.Fatalf("Fstatat: %v", err)
-	}
-
-	if err := unix.Renameat2(f1, "file", f2, "file", unix.RENAME_EXCHANGE); err != nil {
-		t.Errorf("rename EXCHANGE: %v", err)
-	}
-
-	var after1, after2 unix.Stat_t
-	if err := unix.Fstatat(f1, "file", &after1, 0); err != nil {
-		t.Fatalf("Fstatat: %v", err)
-	}
-	if err := unix.Fstatat(f2, "file", &after2, 0); err != nil {
-		t.Fatalf("Fstatat: %v", err)
-	}
-	clearCtime := func(s *unix.Stat_t) {
-		s.Ctim.Sec = 0
-		s.Ctim.Nsec = 0
-	}
-
-	clearCtime(&after1)
-	clearCtime(&after2)
-	clearCtime(&before2)
-	clearCtime(&before1)
-	if diff := pretty.Compare(after1, before2); diff != "" {
-		t.Errorf("after1, before2: %s", diff)
-	}
-	if !reflect.DeepEqual(after2, before1) {
-		t.Errorf("after2, before1: %#v, %#v", after2, before1)
-	}
-
-	root := tc.loopback.EmbeddedInode().Root()
-	ino1 := root.GetChild("file")
-	if ino1 == nil {
-		t.Fatalf("root.GetChild(%q): null inode", "file")
-	}
-	ino2 := root.GetChild("dir").GetChild("file")
-	if ino2 == nil {
-		t.Fatalf("dir.GetChild(%q): null inode", "file")
-	}
-	if ino1.StableAttr().Ino != after1.Ino {
-		t.Errorf("got inode %d for %q, want %d", ino1.StableAttr().Ino, "file", after1.Ino)
-	}
-	if ino2.StableAttr().Ino != after2.Ino {
-		t.Errorf("got inode %d for %q want %d", ino2.StableAttr().Ino, "dir/file", after2.Ino)
-	}
-}
-
 func TestRenameNoOverwrite(t *testing.T) {
 	tc := newTestCase(t, &testOptions{attrCache: true, entryCache: true})
 
@@ -119,63 +45,6 @@ func TestRenameNoOverwrite(t *testing.T) {
 	}
 }
 
-func TestXAttr(t *testing.T) {
-	tc := newTestCase(t, &testOptions{attrCache: true, entryCache: true})
-
-	tc.writeOrig("file", "", 0644)
-
-	buf := make([]byte, 1024)
-	attr := "user.xattrtest"
-	if _, err := syscall.Getxattr(tc.mntDir+"/file", attr, buf); err == syscall.ENOTSUP {
-		t.Skip("$TMP does not support xattrs. Rerun this test with a $TMPDIR override")
-	}
-
-	if _, err := syscall.Getxattr(tc.mntDir+"/file", attr, buf); err != syscall.ENODATA {
-		t.Fatalf("got %v want ENOATTR", err)
-	}
-	value := []byte("value")
-	if err := syscall.Setxattr(tc.mntDir+"/file", attr, value, 0); err != nil {
-		t.Fatalf("Setxattr: %v", err)
-	}
-
-	sz, err := syscall.Listxattr(tc.mntDir+"/file", nil)
-	if err != nil {
-		t.Fatalf("Listxattr: %v", err)
-	}
-	buf = make([]byte, sz)
-	if _, err := syscall.Listxattr(tc.mntDir+"/file", buf); err != nil {
-		t.Fatalf("Listxattr: %v", err)
-	} else {
-		attributes := bytes.Split(buf[:sz], []byte{0})
-		found := false
-		for _, a := range attributes {
-			if string(a) == attr {
-				found = true
-				break
-			}
-		}
-
-		if !found {
-			t.Fatalf("Listxattr: %q (not found: %q", buf[:sz], attr)
-		}
-	}
-
-	sz, err = syscall.Getxattr(tc.mntDir+"/file", attr, buf)
-	if err != nil {
-		t.Fatalf("Getxattr: %v", err)
-	}
-	if bytes.Compare(buf[:sz], value) != 0 {
-		t.Fatalf("Getxattr got %q want %q", buf[:sz], value)
-	}
-	if err := syscall.Removexattr(tc.mntDir+"/file", attr); err != nil {
-		t.Fatalf("Removexattr: %v", err)
-	}
-
-	if _, err := syscall.Getxattr(tc.mntDir+"/file", attr, buf); err != syscall.ENODATA {
-		t.Fatalf("got %v want ENOATTR", err)
-	}
-}
-
 // TestXAttrSymlink verifies that we did not forget to use Lgetxattr instead
 // of Getxattr. This test is Linux-specific because it depends on the behavoir
 // of the `security` namespace.
@@ -246,7 +115,7 @@ func TestCopyFileRange(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Close dst: %v", err)
 	}
-	c, err := ioutil.ReadFile(tc.mntDir + "/dst")
+	c, err := os.ReadFile(tc.mntDir + "/dst")
 	if err != nil {
 		t.Fatalf("ReadFile: %v", err)
 	}
@@ -284,7 +153,7 @@ func waitMount(mnt string) error {
 		if err != nil {
 			return err
 		}
-		content, err := ioutil.ReadFile("/proc/self/mounts")
+		content, err := os.ReadFile("/proc/self/mounts")
 		if err != nil {
 			return err
 		}
@@ -405,3 +274,50 @@ func TestDirectMount(t *testing.T) {
 	}
 	newTestCase(t, opts)
 }
+
+const FS_NOATIME_FL = 0x00000080
+const FS_IOC_GETFLAGS = 0x80086601
+const FS_IOC_SETFLAGS = 0x40086602
+
+func flipNoAtime(fd uintptr) (bool, error) {
+	var flags uint32
+	_, _, errno := syscall.Syscall(syscall.SYS_IOCTL, fd, FS_IOC_GETFLAGS, uintptr(unsafe.Pointer(&flags)))
+	if errno != 0 {
+		return false, errno
+	}
+
+	before := (flags & FS_NOATIME_FL) != 0
+	flags ^= FS_NOATIME_FL
+	_, _, errno = syscall.Syscall(syscall.SYS_IOCTL, uintptr(fd), FS_IOC_SETFLAGS, uintptr(unsafe.Pointer(&flags)))
+	if errno != 0 {
+		return false, errno
+	}
+
+	return before, nil
+}
+
+func TestIoctlLoopback(t *testing.T) {
+	tc := newTestCase(t, &testOptions{attrCache: true, entryCache: true})
+
+	f, err := os.Create(tc.origDir + "/file")
+	if err != nil {
+		t.Fatal(err)
+	}
+	before, err := flipNoAtime(f.Fd())
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	f.Close()
+
+	f, err = os.Open(tc.mntDir + "/file")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer f.Close()
+	after, err := flipNoAtime(f.Fd())
+
+	if after != !before {
+		t.Fatalf("didn't work: after %v, before %v", after, before)
+	}
+}
