@@ -7,7 +7,6 @@ package fs
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
 	"os/exec"
@@ -25,6 +24,8 @@ import (
 	"github.com/hanwen/go-fuse/v2/fuse"
 	"github.com/hanwen/go-fuse/v2/internal/testutil"
 	"github.com/hanwen/go-fuse/v2/posixtest"
+	"github.com/kylelemons/godebug/pretty"
+	"golang.org/x/sys/unix"
 )
 
 type testCase struct {
@@ -41,7 +42,7 @@ type testCase struct {
 
 // writeOrig writes a file into the backing directory of the loopback mount
 func (tc *testCase) writeOrig(path, content string, mode os.FileMode) {
-	if err := ioutil.WriteFile(filepath.Join(tc.origDir, path), []byte(content), mode); err != nil {
+	if err := os.WriteFile(filepath.Join(tc.origDir, path), []byte(content), mode); err != nil {
 		tc.Fatal(err)
 	}
 }
@@ -61,6 +62,8 @@ type testOptions struct {
 	ro                bool
 	directMount       bool // sets MountOptions.DirectMount
 	directMountStrict bool // sets MountOptions.DirectMountStrict
+	disableSplice     bool // sets MountOptions.DisableSplice
+	idMappedMount     bool // sets MountOptions.IDMappedMount
 }
 
 // newTestCase creates the directories `orig` and `mnt` inside a temporary
@@ -111,6 +114,8 @@ func newTestCase(t *testing.T, opts *testOptions) *testCase {
 		DirectMount:       opts.directMount,
 		DirectMountStrict: opts.directMountStrict,
 		EnableLocks:       opts.enableLocks,
+		DisableSplice:     opts.disableSplice,
+		IDMappedMount:     opts.idMappedMount,
 	}
 	if !opts.suppressDebug {
 		mOpts.Debug = testutil.VerboseTest()
@@ -118,6 +123,9 @@ func newTestCase(t *testing.T, opts *testOptions) *testCase {
 	if opts.ro {
 		mOpts.Options = append(mOpts.Options, "ro")
 	}
+	if opts.idMappedMount {
+		mOpts.Options = append(mOpts.Options, "default_permissions")
+	}
 	tc.server, err = fuse.NewServer(tc.rawFS, tc.mntDir, mOpts)
 	if err != nil {
 		t.Fatal(err)
@@ -218,7 +226,7 @@ func TestReadDirStress(t *testing.T) {
 	// Create 110 entries
 	for i := 0; i < 110; i++ {
 		name := fmt.Sprintf("file%036x", i)
-		if err := ioutil.WriteFile(filepath.Join(tc.mntDir, name), []byte("hello"), 0644); err != nil {
+		if err := os.WriteFile(filepath.Join(tc.mntDir, name), []byte("hello"), 0644); err != nil {
 			t.Fatalf("WriteFile %q: %v", name, err)
 		}
 	}
@@ -339,7 +347,7 @@ func TestMknod(t *testing.T) {
 			var st syscall.Stat_t
 			if err := syscall.Stat(p, &st); err != nil {
 				got := st.Mode &^ 07777
-				if want := mode; got != want {
+				if want := uint(mode); want != uint(got) {
 					t.Fatalf("stat(%s): got %o want %o", nm, got, want)
 				}
 			}
@@ -390,14 +398,22 @@ func TestPosix(t *testing.T) {
 	}
 }
 
+func TestReadDisableSplice(t *testing.T) {
+	tc := newTestCase(t, &testOptions{
+		disableSplice: true,
+	})
+
+	posixtest.FileBasic(t, tc.mntDir)
+}
+
 func TestOpenDirectIO(t *testing.T) {
 	// Apparently, tmpfs does not allow O_DIRECT, so try to create
 	// a test temp directory in /var/tmp.
-	ext4Dir, err := ioutil.TempDir("/var/tmp", "go-fuse.TestOpenDirectIO")
+	ext4Dir, err := os.MkdirTemp("/var/tmp", "go-fuse.TestOpenDirectIO")
 	if err != nil {
 		t.Fatalf("MkdirAll: %v", err)
 	}
-	defer os.RemoveAll(ext4Dir)
+	t.Cleanup(func() { os.RemoveAll(ext4Dir) })
 
 	posixtest.DirectIO(t, ext4Dir)
 	if t.Failed() {
@@ -646,28 +662,38 @@ func TestStaleHardlinks(t *testing.T) {
 	// Disable all caches we can disable
 	tc := newTestCase(t, &testOptions{attrCache: false, entryCache: false})
 
+	// gvfsd-trash sets an inotify watch on mntDir and stat()s every file that is
+	// created, racing with the test logic ( https://github.com/hanwen/go-fuse/issues/478 ).
+	// Use a subdir to prevent that.
+	if err := os.Mkdir(tc.mntDir+"/x", 0755); err != nil {
+		t.Fatal(err)
+	}
+
 	// "link0" is original file
-	link0 := tc.mntDir + "/link0"
-	if fd, err := syscall.Creat(link0, 0600); err != nil {
+	link0 := tc.mntDir + "/x/link0"
+	if fd, err := unix.Open(link0, unix.O_CREAT, 0600); err != nil {
 		t.Fatal(err)
 	} else {
 		syscall.Close(fd)
 	}
 	// Create hardlinks via mntDir
+	t.Logf("create link1...20, pid=%d", os.Getpid())
 	for i := 1; i < 20; i++ {
-		linki := fmt.Sprintf(tc.mntDir+"/link%d", i)
+		linki := fmt.Sprintf(tc.mntDir+"/x/link%d", i)
 		if err := syscall.Link(link0, linki); err != nil {
 			t.Fatal(err)
 		}
 	}
 	// Delete hardlinks via origDir (behind loopback fs's back)
+	t.Log("delete link1...20 behind loopback's back")
 	for i := 1; i < 20; i++ {
-		linki := fmt.Sprintf(tc.origDir+"/link%d", i)
+		linki := fmt.Sprintf(tc.origDir+"/x/link%d", i)
 		if err := syscall.Unlink(linki); err != nil {
 			t.Fatal(err)
 		}
 	}
 	// Try to open link0 via mntDir
+	t.Log("open link0")
 	fd, err := syscall.Open(link0, syscall.O_RDONLY, 0)
 	if err != nil {
 		t.Error(err)
@@ -680,3 +706,167 @@ func TestStaleHardlinks(t *testing.T) {
 func init() {
 	syscall.Umask(0)
 }
+
+func testMountDir(dir string) error {
+	opts := &Options{}
+	opts.Debug = testutil.VerboseTest()
+	server, err := Mount(dir, &Inode{}, opts)
+	if err != nil {
+		return err
+	}
+
+	server.Unmount()
+	server.Wait()
+	return nil
+}
+
+func TestParallelMount(t *testing.T) {
+	before := runtime.GOMAXPROCS(1)
+	defer runtime.GOMAXPROCS(before)
+	// Per default, only 1000 FUSE mounts are allowed, then you get
+	// > /usr/bin/fusermount3: too many FUSE filesystems mounted; mount_max=N can be set in /etc/fuse.conf
+	// Let's stay well below 1000.
+	N := 100
+	todo := make(chan string, N)
+	result := make(chan error, N)
+	for i := 0; i < N; i++ {
+		todo <- t.TempDir()
+	}
+	close(todo)
+
+	P := 2
+	for i := 0; i < P; i++ {
+		go func() {
+			for d := range todo {
+				result <- testMountDir(d)
+			}
+		}()
+	}
+
+	for i := 0; i < N; i++ {
+		e := <-result
+		if e != nil {
+			t.Error(e)
+		}
+	}
+}
+
+type handleLessCreateNode struct {
+	Inode
+}
+
+var _ = (NodeCreater)((*handleLessCreateNode)(nil))
+
+func (n *handleLessCreateNode) Create(ctx context.Context, name string, flags uint32, mode uint32, out *fuse.EntryOut) (node *Inode, fh FileHandle, fuseFlags uint32, errno syscall.Errno) {
+	f := &MemRegularFile{
+		Attr: fuse.Attr{
+			Mode: mode,
+		},
+	}
+	ch := n.NewPersistentInode(ctx, f, StableAttr{Mode: fuse.S_IFREG})
+	n.AddChild(name, ch, true)
+	return ch, nil, fuse.FOPEN_KEEP_CACHE, 0
+}
+
+func TestHandleLessCreate(t *testing.T) {
+	hlcn := &handleLessCreateNode{}
+	dir, _ := testMount(t, hlcn, nil)
+
+	posixtest.FileBasic(t, dir)
+}
+
+func lstatxPath(p string) (*unix.Statx_t, error) {
+	var r unix.Statx_t
+	err := unix.Statx(unix.AT_FDCWD, p, unix.AT_SYMLINK_NOFOLLOW,
+		(unix.STATX_BASIC_STATS | unix.STATX_BTIME |
+			unix.STATX_MNT_ID), // unix.STATX_DIOALIGN
+		&r)
+	return &r, err
+}
+
+func clearStatx(st *unix.Statx_t, mask uint32) {
+	st.Mask = mask
+	if mask&(unix.STATX_TYPE|unix.STATX_MODE) == 0 {
+		st.Mode = 0
+	}
+	if mask&(unix.STATX_NLINK) == 0 {
+		st.Nlink = 0
+	}
+	if mask&(unix.STATX_INO) == 0 {
+		st.Ino = 0
+	}
+	if mask&(unix.STATX_ATIME) == 0 {
+		st.Atime = unix.StatxTimestamp{}
+	}
+	if mask&(unix.STATX_BTIME) == 0 {
+		st.Btime = unix.StatxTimestamp{}
+	}
+	if mask&(unix.STATX_CTIME) == 0 {
+		st.Ctime = unix.StatxTimestamp{}
+	}
+	if mask&(unix.STATX_MTIME) == 0 {
+		st.Mtime = unix.StatxTimestamp{}
+	}
+
+	st.Dev_minor = 0
+	st.Dev_major = 0
+	st.Mnt_id = 0
+	st.Attributes_mask = 0
+}
+
+func TestStatx(t *testing.T) {
+	tc := newTestCase(t, &testOptions{attrCache: false, entryCache: false})
+	if err := os.WriteFile(tc.origDir+"/file", []byte("blabla"), 0644); err != nil {
+		t.Fatal(err)
+	}
+
+	oFile := tc.origDir + "/file"
+	want, err := lstatxPath(oFile)
+	if err != nil {
+		t.Fatal(err)
+	}
+	mFile := tc.mntDir + "/file"
+	got, err := lstatxPath(mFile)
+	if err != nil {
+		t.Fatal(err)
+	}
+	mask := got.Mask & want.Mask
+	clearStatx(got, mask)
+	clearStatx(want, mask)
+	if diff := pretty.Compare(got, want); diff != "" {
+		t.Errorf("got, want: %s", diff)
+	}
+
+	// the following works, but does not set Fh in StatxIn
+	oFD, err := os.Open(oFile)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer oFD.Close()
+	mFD, err := os.Open(mFile)
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer mFD.Close()
+
+	var osx, msx unix.Statx_t
+	if err := unix.Statx(int(oFD.Fd()), "", unix.AT_EMPTY_PATH,
+		(unix.STATX_BASIC_STATS | unix.STATX_BTIME |
+			unix.STATX_MNT_ID), // unix.STATX_DIOALIGN
+		&osx); err != nil {
+		t.Fatal(err)
+	}
+	if err := unix.Statx(int(mFD.Fd()), "", unix.AT_EMPTY_PATH,
+		(unix.STATX_BASIC_STATS | unix.STATX_BTIME |
+			unix.STATX_MNT_ID), // unix.STATX_DIOALIGN
+		&msx); err != nil {
+		t.Fatal(err)
+	}
+
+	mask = msx.Mask & osx.Mask
+	clearStatx(&msx, mask)
+	clearStatx(&osx, mask)
+	if diff := pretty.Compare(msx, osx); diff != "" {
+		t.Errorf("got, want: %s", diff)
+	}
+}
