@@ -5,7 +5,7 @@
 package pathfs
 
 import (
-	"io/ioutil"
+	"os"
 	"testing"
 )
 
@@ -18,7 +18,7 @@ func TestCopyFile(t *testing.T) {
 
 	content1 := "blabla"
 
-	err := ioutil.WriteFile(d1+"/file", []byte(content1), 0644)
+	err := os.WriteFile(d1+"/file", []byte(content1), 0644)
 	if err != nil {
 		t.Fatalf("WriteFile failed: %v", err)
 	}
@@ -28,14 +28,14 @@ func TestCopyFile(t *testing.T) {
 		t.Fatal("Unexpected ret code", code)
 	}
 
-	data, err := ioutil.ReadFile(d2 + "/file")
+	data, err := os.ReadFile(d2 + "/file")
 	if content1 != string(data) {
 		t.Fatal("Unexpected content", string(data))
 	}
 
 	content2 := "foobar"
 
-	err = ioutil.WriteFile(d2+"/file", []byte(content2), 0644)
+	err = os.WriteFile(d2+"/file", []byte(content2), 0644)
 	if err != nil {
 		t.Fatalf("WriteFile failed: %v", err)
 	}
@@ -46,7 +46,7 @@ func TestCopyFile(t *testing.T) {
 		t.Fatal("Unexpected ret code", code)
 	}
 
-	data, err = ioutil.ReadFile(d1 + "/file")
+	data, err = os.ReadFile(d1 + "/file")
 	if content2 != string(data) {
 		t.Fatal("Unexpected content", string(data))
 	}
