@@ -9,44 +9,77 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"path/filepath"
+	"reflect"
 	"runtime"
+	"sync"
 	"syscall"
 	"testing"
 
 	"github.com/hanwen/go-fuse/v2/fuse"
+	"github.com/hanwen/go-fuse/v2/internal/fallocate"
+	"github.com/hanwen/go-fuse/v2/internal/xattr"
 	"golang.org/x/sys/unix"
 )
 
 // All holds a map of all test functions
 var All = map[string]func(*testing.T, string){
 	"AppendWrite":                AppendWrite,
-	"SymlinkReadlink":            SymlinkReadlink,
-	"FileBasic":                  FileBasic,
-	"TruncateFile":               TruncateFile,
-	"TruncateNoFile":             TruncateNoFile,
+	"DirSeek":                    DirSeek,
+	"DirectIO":                   DirectIO,
+	"Fallocate":                  Fallocate,
+	"FallocateKeepSize":          FallocateKeepSize,
+	"FcntlFlockLocksFile":        FcntlFlockLocksFile,
+	"FcntlFlockSetLk":            FcntlFlockSetLk,
 	"FdLeak":                     FdLeak,
-	"MkdirRmdir":                 MkdirRmdir,
-	"NlinkZero":                  NlinkZero,
+	"FileBasic":                  FileBasic,
 	"FstatDeleted":               FstatDeleted,
-	"ParallelFileOpen":           ParallelFileOpen,
 	"Link":                       Link,
 	"LinkUnlinkRename":           LinkUnlinkRename,
-	"LseekHoleSeeksToEOF":        LseekHoleSeeksToEOF,
 	"LseekEnxioCheck":            LseekEnxioCheck,
-	"RenameOverwriteDestNoExist": RenameOverwriteDestNoExist,
-	"RenameOverwriteDestExist":   RenameOverwriteDestExist,
-	"RenameOpenDir":              RenameOpenDir,
-	"ReadDir":                    ReadDir,
-	"ReadDirPicksUpCreate":       ReadDirPicksUpCreate,
-	"DirectIO":                   DirectIO,
+	"LseekHoleSeeksToEOF":        LseekHoleSeeksToEOF,
+	"MkdirRmdir":                 MkdirRmdir,
+	"NlinkZero":                  NlinkZero,
 	"OpenAt":                     OpenAt,
-	"Fallocate":                  Fallocate,
-	"DirSeek":                    DirSeek,
-	"FcntlFlockSetLk":            FcntlFlockSetLk,
-	"FcntlFlockLocksFile":        FcntlFlockLocksFile,
+	"OpenSymlinkRace":            OpenSymlinkRace,
+	"ParallelFileOpen":           ParallelFileOpen,
+	"ReadDir":                    ReadDir,
+	"ReadDirConsistency":         ReadDirConsistency,
+	"RenameOpenDir":              RenameOpenDir,
+	"RenameOverwriteDestExist":   RenameOverwriteDestExist,
+	"RenameOverwriteDestNoExist": RenameOverwriteDestNoExist,
+	"SetattrSymlink":             SetattrSymlink,
+	"SymlinkReadlink":            SymlinkReadlink,
+	"TruncateFile":               TruncateFile,
+	"TruncateNoFile":             TruncateNoFile,
+	"XAttr":                      XAttr,
+}
+
+func SetattrSymlink(t *testing.T, mnt string) {
+	l := filepath.Join(mnt, "link")
+	if err := os.Symlink("doesnotexist", l); err != nil {
+		t.Fatalf("symlink: %v", err)
+	}
+
+	tvs := []unix.Timeval{
+		{Sec: 42, Usec: 1},
+		{Sec: 43, Usec: 2},
+	}
+	if err := unix.Lutimes(l, tvs); err != nil {
+		t.Fatalf("Lutimes: %v", err)
+	}
+
+	var st unix.Stat_t
+	if err := unix.Lstat(l, &st); err != nil {
+		t.Fatalf("Lstat: %v", err)
+	}
+
+	if st.Mtim.Sec != 43 {
+		// Can't check atime; it's hard to prevent implicit readlink calls.
+		t.Fatalf("got mtime %v, want 43", st.Mtim)
+	}
 }
 
 func DirectIO(t *testing.T, mnt string) {
@@ -109,11 +142,11 @@ func FileBasic(t *testing.T, mnt string) {
 	content := []byte("hello world")
 	fn := mnt + "/file"
 
-	if err := ioutil.WriteFile(fn, content, 0755); err != nil {
+	if err := os.WriteFile(fn, content, 0755); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
-	if got, err := ioutil.ReadFile(fn); err != nil {
+	if got, err := os.ReadFile(fn); err != nil {
 		t.Fatalf("ReadFile: %v", err)
 	} else if bytes.Compare(got, content) != 0 {
 		t.Errorf("ReadFile: got %q, want %q", got, content)
@@ -146,7 +179,7 @@ func FileBasic(t *testing.T, mnt string) {
 func TruncateFile(t *testing.T, mnt string) {
 	content := []byte("hello world")
 	fn := mnt + "/file"
-	if err := ioutil.WriteFile(fn, content, 0755); err != nil {
+	if err := os.WriteFile(fn, content, 0755); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -165,7 +198,7 @@ func TruncateFile(t *testing.T, mnt string) {
 		t.Errorf("Close: %v", err)
 	}
 
-	if got, err := ioutil.ReadFile(fn); err != nil {
+	if got, err := os.ReadFile(fn); err != nil {
 		t.Fatalf("ReadFile: %v", err)
 	} else if want := content[:trunc]; bytes.Compare(got, want) != 0 {
 		t.Errorf("got %q, want %q", got, want)
@@ -173,7 +206,7 @@ func TruncateFile(t *testing.T, mnt string) {
 }
 func TruncateNoFile(t *testing.T, mnt string) {
 	fn := mnt + "/file"
-	if err := ioutil.WriteFile(fn, []byte("hello"), 0644); err != nil {
+	if err := os.WriteFile(fn, []byte("hello"), 0644); err != nil {
 		t.Errorf("WriteFile: %v", err)
 	}
 
@@ -192,12 +225,12 @@ func TruncateNoFile(t *testing.T, mnt string) {
 func FdLeak(t *testing.T, mnt string) {
 	fn := mnt + "/file"
 
-	if err := ioutil.WriteFile(fn, []byte("hello world"), 0755); err != nil {
+	if err := os.WriteFile(fn, []byte("hello world"), 0755); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
 	for i := 0; i < 100; i++ {
-		if _, err := ioutil.ReadFile(fn); err != nil {
+		if _, err := os.ReadFile(fn); err != nil {
 			t.Fatalf("ReadFile: %v", err)
 		}
 	}
@@ -231,11 +264,11 @@ func MkdirRmdir(t *testing.T, mnt string) {
 func NlinkZero(t *testing.T, mnt string) {
 	src := mnt + "/src"
 	dst := mnt + "/dst"
-	if err := ioutil.WriteFile(src, []byte("source"), 0644); err != nil {
+	if err := os.WriteFile(src, []byte("source"), 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
-	if err := ioutil.WriteFile(dst, []byte("dst"), 0644); err != nil {
+	if err := os.WriteFile(dst, []byte("dst"), 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -272,19 +305,19 @@ func FstatDeleted(t *testing.T, mnt string) {
 	const iMax = 9
 	type file struct {
 		fd int
-		st syscall.Stat_t
+		st unix.Stat_t
 	}
 	files := make(map[int]file)
 	for i := 0; i <= iMax; i++ {
 		// Create files with different sizes
 		path := fmt.Sprintf("%s/%d", mnt, i)
 		content := make([]byte, i)
-		err := ioutil.WriteFile(path, content, 0644)
+		err := os.WriteFile(path, content, 0644)
 		if err != nil {
 			t.Fatalf("WriteFile: %v", err)
 		}
-		var st syscall.Stat_t
-		err = syscall.Stat(path, &st)
+		var st unix.Stat_t
+		err = unix.Stat(path, &st)
 		if err != nil {
 			t.Fatal(err)
 		}
@@ -303,14 +336,14 @@ func FstatDeleted(t *testing.T, mnt string) {
 	}
 	// Fstat in random order
 	for _, v := range files {
-		var st syscall.Stat_t
-		err := syscall.Fstat(v.fd, &st)
+		var st unix.Stat_t
+		err := unix.Fstat(v.fd, &st)
 		if err != nil {
 			t.Fatal(err)
 		}
 		// Ignore ctime, changes on unlink
-		v.st.Ctim = syscall.Timespec{}
-		st.Ctim = syscall.Timespec{}
+		v.st.Ctim = unix.Timespec{}
+		st.Ctim = unix.Timespec{}
 		// Nlink value should have dropped to zero
 		v.st.Nlink = 0
 		// Rest should stay the same
@@ -322,7 +355,7 @@ func FstatDeleted(t *testing.T, mnt string) {
 
 func ParallelFileOpen(t *testing.T, mnt string) {
 	fn := mnt + "/file"
-	if err := ioutil.WriteFile(fn, []byte("content"), 0644); err != nil {
+	if err := os.WriteFile(fn, []byte("content"), 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -356,7 +389,7 @@ func Link(t *testing.T, mnt string) {
 	link := mnt + "/link"
 	target := mnt + "/target"
 
-	if err := ioutil.WriteFile(target, []byte("hello"), 0644); err != nil {
+	if err := os.WriteFile(target, []byte("hello"), 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -395,12 +428,12 @@ func RenameOverwrite(t *testing.T, mnt string, destExists bool) {
 	if err := os.Mkdir(mnt+"/dir", 0755); err != nil {
 		t.Fatalf("Mkdir: %v", err)
 	}
-	if err := ioutil.WriteFile(mnt+"/file", []byte("hello"), 0644); err != nil {
+	if err := os.WriteFile(mnt+"/file", []byte("hello"), 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
 	if destExists {
-		if err := ioutil.WriteFile(mnt+"/dir/renamed", []byte("xx"), 0644); err != nil {
+		if err := os.WriteFile(mnt+"/dir/renamed", []byte("xx"), 0644); err != nil {
 			t.Fatalf("WriteFile dest: %v", err)
 		}
 	}
@@ -466,6 +499,28 @@ func RenameOpenDir(t *testing.T, mnt string) {
 	}
 }
 
+func readAllDirEntries(fd int) ([]fuse.DirEntry, error) {
+	var buf [4096]byte
+	var result []fuse.DirEntry
+	for {
+		n, err := unix.ReadDirent(fd, buf[:])
+		if err != nil {
+			return nil, err
+		}
+		if n == 0 {
+			break
+		}
+		todo := buf[:n]
+		for len(todo) > 0 {
+			var de fuse.DirEntry
+			n := de.Parse(todo)
+			todo = todo[n:]
+			result = append(result, de)
+		}
+	}
+	return result, nil
+}
+
 // ReadDir creates 110 files one by one, checking that we get the expected
 // entries after each file creation.
 func ReadDir(t *testing.T, mnt string) {
@@ -475,7 +530,7 @@ func ReadDir(t *testing.T, mnt string) {
 	for i := 0; i < 110; i++ {
 		nm := fmt.Sprintf("file%036x", i)
 		want[nm] = true
-		if err := ioutil.WriteFile(filepath.Join(mnt, nm), []byte("hello"), 0644); err != nil {
+		if err := os.WriteFile(filepath.Join(mnt, nm), []byte("hello"), 0644); err != nil {
 			t.Fatalf("WriteFile %q: %v", nm, err)
 		}
 		// Verify that we get the expected entries
@@ -508,24 +563,24 @@ func ReadDir(t *testing.T, mnt string) {
 	}
 }
 
-// Readdir should pick file created after open, but before readdir.
-func ReadDirPicksUpCreate(t *testing.T, mnt string) {
-	f, err := os.Open(mnt)
-	if err != nil {
-		t.Fatalf("Open: %v", err)
-	}
+// Creates files, read them back using readdir
+func ReadDirConsistency(t *testing.T, mnt string) {
+	var results [][]fuse.DirEntry
+	for i := 0; i < 2; i++ {
+		fd, err := syscall.Open(mnt, syscall.O_DIRECTORY, 0)
+		if err != nil {
+			t.Fatalf("Open: %v", err)
+		}
+		defer syscall.Close(fd)
 
-	if err := ioutil.WriteFile(mnt+"/file", []byte{42}, 0644); err != nil {
-		t.Fatalf("WriteFile: %v", err)
+		r, err := readAllDirEntries(fd)
+		if err != nil {
+			t.Fatalf("ReadDir: %v", err)
+		}
+		results = append(results, r)
 	}
-	names, err := f.Readdirnames(-1)
-	if err != nil {
-		t.Fatalf("ReadDir: %v", err)
-	}
-	f.Close()
-
-	if len(names) != 1 || names[0] != "file" {
-		t.Errorf("missing file created after opendir")
+	if !reflect.DeepEqual(results[0], results[1]) {
+		t.Errorf("got %v, want %v", results[0], results[1])
 	}
 }
 
@@ -533,7 +588,7 @@ func ReadDirPicksUpCreate(t *testing.T, mnt string) {
 func LinkUnlinkRename(t *testing.T, mnt string) {
 	content := []byte("hello")
 	tmp := mnt + "/tmpfile"
-	if err := ioutil.WriteFile(tmp, content, 0644); err != nil {
+	if err := os.WriteFile(tmp, content, 0644); err != nil {
 		t.Fatalf("WriteFile %q: %v", tmp, err)
 	}
 
@@ -545,7 +600,7 @@ func LinkUnlinkRename(t *testing.T, mnt string) {
 		t.Fatalf("Unlink %q: %v", tmp, err)
 	}
 
-	if back, err := ioutil.ReadFile(dest); err != nil {
+	if back, err := os.ReadFile(dest); err != nil {
 		t.Fatalf("Read %q: %v", dest, err)
 	} else if bytes.Compare(back, content) != 0 {
 		t.Fatalf("Read got %q want %q", back, content)
@@ -577,7 +632,7 @@ func AppendWrite(t *testing.T, mnt string) {
 	fd = 0
 	want := []byte("helloworld")
 
-	got, err := ioutil.ReadFile(mnt + "/file")
+	got, err := os.ReadFile(mnt + "/file")
 	if err != nil {
 		t.Fatalf("ReadFile: %v", err)
 	}
@@ -607,7 +662,7 @@ func OpenAt(t *testing.T, mnt string) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	fd, err := syscall.Openat(dirfd, "file1", syscall.O_CREAT, 0700)
+	fd, err := unix.Openat(dirfd, "file1", syscall.O_CREAT, 0700)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -624,7 +679,7 @@ func Fallocate(t *testing.T, mnt string) {
 		t.Fatalf("OpenFile failed: %v", err)
 	}
 	defer rwFile.Close()
-	err = syscall.Fallocate(int(rwFile.Fd()), 0, 1024, 4096)
+	err = fallocate.Fallocate(int(rwFile.Fd()), 0, 1024, 4096)
 	if err != nil {
 		t.Fatalf("Fallocate failed: %v", err)
 	}
@@ -638,6 +693,31 @@ func Fallocate(t *testing.T, mnt string) {
 	}
 }
 
+func FallocateKeepSize(t *testing.T, mnt string) {
+	f, err := os.Create(mnt + "/file")
+	if err != nil {
+		t.Fatal(err)
+	}
+	defer f.Close()
+	data := bytes.Repeat([]byte{42}, 100)
+	if _, err := f.Write(data); err != nil {
+		t.Fatal(err)
+	}
+
+	if err := syscall.Fallocate(int(f.Fd()), unix.FALLOC_FL_KEEP_SIZE, 50, 52); err != nil {
+		t.Fatal(err)
+	}
+
+	if _, err := f.Seek(0, io.SeekStart); err != nil {
+		t.Fatal(err)
+	}
+
+	roundtrip, _ := io.ReadAll(f)
+	if !bytes.Equal(roundtrip, data) {
+		t.Fatalf("roundtrip not equal %q != %q", roundtrip, data)
+	}
+}
+
 func FcntlFlockSetLk(t *testing.T, mnt string) {
 	for i, cmd := range []int{syscall.F_SETLK, syscall.F_SETLKW} {
 		filename := mnt + fmt.Sprintf("/file%d", i)
@@ -661,12 +741,11 @@ func FcntlFlockSetLk(t *testing.T, mnt string) {
 		}
 		defer f2.Close()
 		lk := syscall.Flock_t{}
-		if err := syscall.FcntlFlock(f2.Fd(), unix.F_OFD_GETLK, &lk); err != nil {
+		if err := sysFcntlFlockGetOFDLock(f2.Fd(), &lk); err != nil {
 			t.Errorf("FcntlFlock failed: %v", err)
 		}
 		if lk.Type != syscall.F_WRLCK {
 			t.Errorf("got lk.Type=%v, want %v", lk.Type, syscall.F_WRLCK)
-
 		}
 	}
 }
@@ -705,7 +784,7 @@ func FcntlFlockLocksFile(t *testing.T, mnt string) {
 func LseekHoleSeeksToEOF(t *testing.T, mnt string) {
 	fn := filepath.Join(mnt, "file.bin")
 	content := bytes.Repeat([]byte("abcxyz\n"), 1024)
-	if err := ioutil.WriteFile(fn, content, 0644); err != nil {
+	if err := os.WriteFile(fn, content, 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -726,7 +805,7 @@ func LseekHoleSeeksToEOF(t *testing.T, mnt string) {
 func LseekEnxioCheck(t *testing.T, mnt string) {
 	fn := filepath.Join(mnt, "file.bin")
 	content := bytes.Repeat([]byte("abcxyz\n"), 1024)
-	if err := ioutil.WriteFile(fn, content, 0644); err != nil {
+	if err := os.WriteFile(fn, content, 0644); err != nil {
 		t.Fatalf("WriteFile: %v", err)
 	}
 
@@ -767,3 +846,205 @@ func LseekEnxioCheck(t *testing.T, mnt string) {
 		}
 	}
 }
+
+// XAttr; they aren't posix but cross-platform enough to put into posixtest/
+func XAttr(t *testing.T, mntDir string) {
+	buf := make([]byte, 1024)
+	attrNameSpace := "user"
+	attrName := "xattrtest"
+	attr := fmt.Sprintf("%s.%s", attrNameSpace, attrName)
+	fn := mntDir + "/file"
+
+	if err := os.WriteFile(fn, []byte{}, 0666); err != nil {
+		t.Fatalf("WriteFile: %v", err)
+	}
+
+	if _, err := unix.Getxattr(fn, attr, buf); err == unix.ENOTSUP {
+		t.Skipf("filesystem for %s does not support xattrs. Rerun this test with a $TMPDIR override", fn)
+	}
+
+	if _, err := unix.Getxattr(fn, attr, buf); err != xattr.ENOATTR {
+		t.Fatalf("got %v want ENOATTR", err)
+	}
+	value := []byte("value")
+	if err := unix.Setxattr(fn, attr, value, 0); err != nil {
+		t.Fatalf("Setxattr: %v", err)
+	}
+
+	sz, err := unix.Listxattr(fn, nil)
+	if err != nil {
+		t.Fatalf("Listxattr: %v", err)
+	}
+	buf = make([]byte, sz)
+	if _, err := unix.Listxattr(fn, buf); err != nil {
+		t.Fatalf("Listxattr: %v", err)
+	} else {
+		attributes := xattr.ParseAttrNames(buf[:sz])
+		found := false
+		for _, a := range attributes {
+			if string(a) == attr || attrNameSpace+string(a) == attr {
+				found = true
+				break
+			}
+		}
+
+		if !found {
+			t.Fatalf("Listxattr: %q (not found: %q", attributes, attr)
+		}
+	}
+
+	sz, err = unix.Getxattr(fn, attr, buf)
+	if err != nil {
+		t.Fatalf("Getxattr: %v", err)
+	}
+	if bytes.Compare(buf[:sz], value) != 0 {
+		t.Fatalf("Getxattr got %q want %q", buf[:sz], value)
+	}
+	if err := unix.Removexattr(fn, attr); err != nil {
+		t.Fatalf("Removexattr: %v", err)
+	}
+
+	if _, err := unix.Getxattr(fn, attr, buf); err != xattr.ENOATTR {
+		t.Fatalf("got %v want ENOATTR", err)
+	}
+}
+
+// Test if Open() is vulnerable to symlink-race attacks using two goroutines:
+//
+// goroutine "shuffler":
+// In a loop:
+// * Replace empty file "OpenSymlinkRace" with a symlink pointing to /etc/passwd
+// * Replace back with empty file
+//
+// goroutine "opener":
+// In a loop:
+// * Open "OpenSymlinkRace" and call Fstat on it. Now there's three cases:
+//  1. Size=0: we opened the empty file created by shuffler. Normal and uninteresting.
+//  2. Size>0 but Dev number different: we (this test) opened /etc/passwd ourselves
+//     because we resolved the symlink. Normal.
+//  3. Size>0 and Dev number matches the FUSE mount: go-fuse opened /etc/passwd.
+//     The attack has worked.
+func OpenSymlinkRace(t *testing.T, mnt string) {
+	path := mnt + "/OpenSymlinkRace"
+
+	var wg sync.WaitGroup
+	wg.Add(2)
+
+	const iterations = 1000
+
+	fd, err := syscall.Creat(path, 0600)
+	if err != nil {
+		t.Error(err)
+		return
+	}
+	// Find and save the device number of the FUSE mount
+	var st syscall.Stat_t
+	err = syscall.Fstat(fd, &st)
+	fuseMountDev := st.Dev
+	if err != nil {
+		t.Fatal(err)
+	}
+	syscall.Close(fd)
+
+	// Shuffler
+	go func() {
+		defer wg.Done()
+		tmp := path + ".tmp"
+		for i := 0; i < iterations; i++ {
+			// Stop when another thread has failed
+			if t.Failed() {
+				return
+			}
+
+			// Make "path" a regular file
+			fd, err := syscall.Creat(tmp, 0600)
+			if err != nil {
+				t.Errorf("shuffler: Creat: %v", err)
+				return
+			}
+			syscall.Close(fd)
+			err = syscall.Rename(tmp, path)
+			if err != nil {
+				t.Error(err)
+				return
+			}
+
+			// Make "path" a symlink
+			err = syscall.Symlink("/etc/passwd", tmp)
+			if err != nil {
+				t.Errorf("shuffler: Symlink: %v", err)
+				return
+			}
+			err = syscall.Rename(tmp, path)
+			if err != nil {
+				t.Errorf("shuffler: Rename: %v", err)
+				return
+			}
+		}
+	}()
+
+	// Keep some statistics
+	type statsT struct {
+		EINVAL          int
+		ENOENT          int
+		ELOOP           int
+		empty           int
+		resolvedSymlink int
+	}
+	var stats statsT
+
+	// Opener
+	go func() {
+		defer wg.Done()
+		var st syscall.Stat_t
+
+		for i := 0; i < iterations; i++ {
+			// Stop when another thread has failed
+			if t.Failed() {
+				return
+			}
+
+			fd, err := syscall.Open(path, syscall.O_RDONLY, 0)
+			if err != nil {
+				if err == syscall.EINVAL {
+					// Happens when the kernel tries to read the symlink but
+					// it's already a file again in the backing directory
+					stats.EINVAL++
+					continue
+				}
+				if err == syscall.ELOOP {
+					// Looks like there's some symlink-safety
+					stats.ELOOP++
+					continue
+				}
+				if err == syscall.ENOENT {
+					// Not sure why we get these, but we do
+					stats.ENOENT++
+					continue
+				}
+				t.Errorf("opener: Open: %v", err)
+				return
+			}
+			err = syscall.Fstat(fd, &st)
+			syscall.Close(fd)
+			if err != nil {
+				t.Errorf("opener: Fstat: %v", err)
+				return
+			}
+			if st.Size == 0 {
+				stats.empty++
+				continue
+			}
+			if st.Dev != fuseMountDev {
+				stats.resolvedSymlink++
+			} else {
+				// opened /etc/passwd
+				t.Errorf("opener: successful symlink attack in iteration %d. We tricked go-fuse into opening /etc/passwd.", i)
+				return
+			}
+		}
+	}()
+
+	wg.Wait()
+	t.Logf("opener stats: %#v", stats)
+}
